CREATE TABLE global (
  id            INTEGER NOT NULL DEFAULT 1 PRIMARY KEY CHECK (id IS 1),
  last_post     INTEGER NOT NULL DEFAULT 0,
  last_file_ref INTEGER NOT NULL DEFAULT 0,
  last_action   INTEGER NOT NULL DEFAULT 0,
  cooldown_post       INTEGER NOT NULL DEFAULT 30,
  cooldown_thread     INTEGER NOT NULL DEFAULT 720,
  burst_post          INTEGER NOT NULL DEFAULT 16,
  burst_thread        INTEGER NOT NULL DEFAULT 4,
  captcha_block_ratio INTEGER NOT NULL DEFAULT 4,
  burst_window_ratio  INTEGER NOT NULL DEFAULT 4
);

CREATE TABLE boards (
  board             TEXT NOT NULL PRIMARY KEY,
  overboard_exclude INTEGER NOT NULL DEFAULT 0,
  mod_ids           INTEGER NOT NULL DEFAULT 0,
  new_threads_disabled INTEGER NOT NULL DEFAULT 0,
  text_only         INTEGER NOT NULL DEFAULT 0
);

CREATE TABLE threads (
  thread         INTEGER NOT NULL PRIMARY KEY,
  board          TEXT    NOT NULL,
  original_board TEXT    NOT NULL,
  max_bump       INTEGER,
  overboard_exclude INTEGER
);

CREATE TABLE posts (
  post            INTEGER NOT NULL PRIMARY KEY,
  thread          INTEGER NOT NULL REFERENCES threads ON UPDATE CASCADE ON DELETE CASCADE,
  original_thread INTEGER NOT NULL,
  moderator       TEXT,
  subject         TEXT    NOT NULL DEFAULT '',
  date            INTEGER NOT NULL,
  comment         TEXT    NOT NULL DEFAULT '',
  overboard_exclude INTEGER NOT NULL DEFAULT 0
);

CREATE TABLE files (
  hash          TEXT    NOT NULL PRIMARY KEY,
  extension     TEXT    NOT NULL DEFAULT '',
  thumb         TEXT,
  thumb_catalog TEXT,
  size          INTEGER,
  width         INTEGER,
  height        INTEGER,
  duration      REAL,
  thumb_width   INTEGER,
  thumb_height  INTEGER,
  type          INTEGER,
  status        TEXT
);

CREATE TABLE file_refs (
  file_ref INTEGER NOT NULL PRIMARY KEY,
  post     INTEGER NOT NULL REFERENCES posts ON UPDATE CASCADE ON DELETE CASCADE,
  position INTEGER NOT NULL,
  hash     TEXT    NOT NULL REFERENCES files ON UPDATE CASCADE ON DELETE CASCADE,
  name     TEXT,
  name_uri TEXT,
  spoiler  TEXT,
  UNIQUE (post, position)
);

CREATE TABLE links (
  src      INTEGER NOT NULL REFERENCES posts ON UPDATE CASCADE ON DELETE CASCADE,
  position INTEGER NOT NULL,
  dest     INTEGER NOT NULL,
  PRIMARY KEY (src, position)
);

CREATE TABLE mod_actions (
  action      INTEGER NOT NULL PRIMARY KEY,
  date        INTEGER NOT NULL,
  moderator   TEXT    NOT NULL,
  function    TEXT,
  arguments   TEXT,
  description TEXT    NOT NULL,
  reason      TEXT,
  status      INTEGER NOT NULL DEFAULT 1
);

CREATE TABLE mod_row_actions (
  action    INTEGER NOT NULL REFERENCES mod_actions ON UPDATE CASCADE ON DELETE CASCADE,
  table_id  INTEGER NOT NULL,
  row_key           NOT NULL,
  is_insert INTEGER NOT NULL,
  is_delete INTEGER NOT NULL,
  PRIMARY KEY (action, table_id, row_key)
);

CREATE TABLE mod_changes (
  action    INTEGER NOT NULL REFERENCES mod_actions ON UPDATE CASCADE ON DELETE CASCADE,
  table_id  INTEGER NOT NULL,
  row_key           NOT NULL,
  column_id TEXT    NOT NULL,
  old_value,
  new_value,
  PRIMARY KEY (action, table_id, row_key, column_id)
);

CREATE INDEX posts_by_thread ON posts (thread, post);
CREATE INDEX posts_by_thread_date ON posts (thread, date, post);
CREATE INDEX posts_by_original_thread ON posts (original_thread, post) WHERE thread <> original_thread;
CREATE INDEX file_refs_by_post ON file_refs (post, position);
CREATE INDEX file_refs_by_hash ON file_refs (hash);
CREATE INDEX links_by_dest ON links (dest, src, position);
CREATE INDEX mod_row_actions_by_log_order ON mod_row_actions (action DESC, table_id, row_key);

CREATE TRIGGER increment_last_post AFTER INSERT ON posts FOR EACH ROW BEGIN
  UPDATE global SET last_post = MAX(last_post, NEW.post);
END;

CREATE TRIGGER increment_last_file_ref AFTER INSERT ON file_refs FOR EACH ROW BEGIN
  UPDATE global SET last_file_ref = MAX(last_file_ref, NEW.file_ref);
END;

CREATE TRIGGER increment_last_action AFTER INSERT ON mod_actions FOR EACH ROW BEGIN
  UPDATE global SET last_action = MAX(last_action, NEW.action);
END;
