local common = {}

function common.read_all(filename)
  local file = assert(io.open(filename))
  local text = assert(file:read('*all'))
  assert(file:close())
  return text
end

function common.handle_errors(f, write)
  local data = {}
  local function write2(s)
    table.insert(data, s)
  end
  local err
  local status = xpcall(
    function() f(write2) end,
    function(message) err = debug.traceback(message, 2) end
  )
  if status then
    for _, s in ipairs(data) do
      write(s)
    end
  else
    io.stderr:write(err .. '\n')
    write('Status: 500 Internal Server Error\r\n')
    write('Content-Type: text/plain; charset=utf-8\r\n\r\n')
    write(err .. '\n')
    os.exit(1)
  end
end

function common.exec_read(command)
  local proc = assert(io.popen(command))
  local data = assert(proc:read('*all'))
  assert(proc:close())
  return data
end

function common.pack(...)
  return {n = select('#', ...), ...}
end

function common.utf8len(s)
  local _, len = s:gsub('[^\128-\191]', '')
  return len
end

function common.utf16len(s)
  local _, astral = s:gsub('[\240-\247]', '')
  return common.utf8len(s) + astral
end

return common
