local login = {}
local openssl = require('openssl')
local config = require('config')
local db = require('database')('../Data/private.db')

local query_password = db:prepare("SELECT moderator, status FROM logins WHERE password = ?")

local alphabet = '123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz'

function login.generate()
  local ndigits = config.PASSWORD_LENGTH
  local base = alphabet:len()
  local nbits = math.ceil(ndigits * math.log(base) / math.log(2)) + 16
  local n = openssl.bn.random(nbits)
  local password = ''
  for _ = 1, ndigits do
    local digit = (n % base):tonumber()
    local c = alphabet:sub(digit + 1, digit + 1)
    n = n / base
    password = password .. c
  end
  return password
end

function login.get_hash(password)
  return openssl.hex(openssl.digest.get('sha256'):digest(password))
end

function login.verify(password)
  local hash = login.get_hash(password)
  return query_password(hash)
end

return login
