#!/usr/bin/luajit
local fcgi = require('fcgi')
local sys_stat = require('posix.sys.stat')
local config = require('config')
local common = require('common')
local db = require('database')('../Data/board.db', true):read('cache.sql')
local captcha = require('captcha')
local gen = require('gen')

local HEADERS = 'Content-Type: text/html; charset=utf-8\r\n\r\n'

local function template(text, defaults)
  return function(options)

    local options2
    if defaults then
      options2 = {}
      for k, v in pairs(defaults) do options2[k] = v end
      for k, v in pairs(options)  do options2[k] = v end
    else
      options2 = options
    end

    local text2 = text
    local n
    repeat
      text2, n = text2:gsub('%?(%w+)(%(?%)?)(%b{})(\n?)', function(test, is_fun, body, newline)
        local test_result = options2[test]
        if is_fun == '()' then
          test_result = test_result()
        end
        if test_result then
          return body:sub(2, -2) .. newline
        else
          return ''
        end
      end)
    until n == 0

    return text2
      :gsub('{(%w+)%(%)}', function(k) return options2[k](options2) end)
      :gsub('{(%w+)}', function(k) return assert(options2[k], 'missing ' .. k) end)

  end
end

--[[
  Order of GROUP_CONCAT is undefined but this works in practice
  because sqlite does not flatten the subquery per:
  https://sqlite.org/optoverview.html#flattening
  Alternatives tried do not perform as well.
]]
local get_thread = db:prepare([[
  SELECT (SELECT html FROM post_cache WHERE thread = ?1 AND post = ?1) || IFNULL((
    SELECT GROUP_CONCAT('<hr class="hidden" />' || html, '') FROM
      (SELECT html FROM post_cache WHERE thread = ?1 AND post <> ?1 ORDER BY post)
  ), '')
]])

local function overboard_type(board)
  return ({All = 1, All2 = 2})[board] or false
end

local get_board = {[false]={}, [1] = {}, [2] = {}}
local get_thread_count = {}
local get_recent_post_count = {}
local get_latest = {}

for _, overboard in ipairs{false, 1, 2} do
  local iob = (overboard or 0) + 1

  get_board[overboard].index = db:prepare([[
    SELECT GROUP_CONCAT('<hr class="hidden" />' || index_html, '') FROM (
      SELECT index_html
        FROM thread_cache AS c
        ]] .. ({
          'WHERE overboard = 0 AND c.board = :board',
          'JOIN threads AS t ON c.thread = t.thread LEFT JOIN boards AS b ON c.board = b.board'
            .. ' WHERE overboard = 1 AND NOT COALESCE(t.overboard_exclude, b.overboard_exclude, 0)',
          'WHERE overboard = 0'
        })[iob] .. [[
        ORDER BY max_post DESC LIMIT :limit OFFSET :offset
    )
  ]])

  get_board[overboard].catalog = db:prepare([[
    SELECT '<div class="catalog-root">'
      || GROUP_CONCAT('<hr class="hidden" />' || html, '') || '</div>' || CHAR(10)
    FROM (
      SELECT REPLACE(catalog_html, '</time>', 'last ' || relative_date || ' ago</time>') AS html
        FROM thread_cache AS c
        JOIN thread_relative_dates AS r ON c.thread = r.thread
        ]] .. ({
          'WHERE overboard = 0 AND c.board = :board',
          'JOIN threads AS t ON c.thread = t.thread LEFT JOIN boards AS b ON c.board = b.board'
            .. ' WHERE overboard = 1 AND NOT COALESCE(t.overboard_exclude, b.overboard_exclude, 0)',
          'WHERE overboard = 0'
        })[iob] .. [[
        ORDER BY max_post DESC LIMIT :limit OFFSET :offset
    )
  ]])

  get_board[overboard].recent = db:prepare([[
    SELECT GROUP_CONCAT('<hr class="hidden" />' || html, '') FROM (
      ]] .. ({
        'SELECT html FROM post_cache WHERE board = :board ORDER BY post DESC LIMIT :limit OFFSET :offset',
        [[
          SELECT html FROM (
              SELECT html, post, thread, board FROM post_cache ORDER BY post DESC LIMIT :limit OFFSET :offset
          ) AS c
            JOIN posts AS p ON c.post = p.post JOIN threads AS t ON c.thread = t.thread
            LEFT JOIN boards AS b ON c.board = b.board
            WHERE NOT p.overboard_exclude AND NOT COALESCE(t.overboard_exclude, b.overboard_exclude, 0)
        ]],
        'SELECT html FROM post_cache ORDER BY post DESC LIMIT :limit OFFSET :offset'
      })[iob] .. [[
    )
  ]])

  get_thread_count[overboard] = db:prepare([[
    SELECT COUNT() FROM thread_cache AS c
      ]] .. ({
        'WHERE overboard = 0 AND c.board = :board',
        'JOIN threads AS t ON c.thread = t.thread LEFT JOIN boards AS b ON c.board = b.board'
          .. ' WHERE overboard = 1 AND NOT COALESCE(t.overboard_exclude, b.overboard_exclude, 0)',
        'WHERE overboard = 0'
      })[iob] .. [[
  ]])

  get_recent_post_count[overboard] = db:prepare([[
    SELECT COUNT() FROM post_cache
      ]] .. (overboard and '' or 'WHERE board = :board') .. [[
  ]])

end

get_latest[false] = db:prepare([[
  SELECT IFNULL(MAX(post), 0) FROM posts JOIN threads ON posts.thread = threads.thread WHERE board = :board
]])
get_latest[1] = db:prepare([[
  SELECT IFNULL(MAX(post), 0) FROM posts JOIN threads ON posts.thread = threads.thread
    LEFT JOIN boards ON threads.board = boards.board
    WHERE NOT posts.overboard_exclude AND NOT COALESCE(threads.overboard_exclude, boards.overboard_exclude, 0)
]])
get_latest[2] = db:prepare([[
  SELECT last_post FROM global
]])

local get_log = db:prepare([[
  SELECT GROUP_CONCAT('<hr class="hidden" />' || html, '') FROM (
    SELECT
      '<fieldset class="modaction">' || mah.html
        || GROUP_CONCAT('<hr class="hidden" />' || t2.html, '') || '</fieldset>' || CHAR(10)
      AS html
    FROM (
      SELECT action,
        '<div>' || CHAR(10)
          || GROUP_CONCAT(html, '')
          || IFNULL(REPLACE(
               (SELECT html FROM post_cache AS c WHERE c.post = t1.post),
               ' id="P' || post,
               ' id="P' || post || '_A' || action
             ), '')
          || '</div>' || CHAR(10)
        AS html
      FROM (
        SELECT action, table_id, row_key, post, html
        FROM mod_row_action_html
        ORDER BY action DESC, table_id, row_key
        LIMIT ? OFFSET ?
      ) AS t1
      GROUP BY action, post
      ORDER BY action DESC, MIN(table_id), MIN(row_key)
    ) AS t2
    JOIN mod_action_html AS mah ON t2.action = mah.action
    GROUP BY t2.action
    ORDER BY t2.action DESC
  )
]])

local get_thread_location =
  db:prepare('SELECT board FROM thread_cache WHERE thread = ?')

local get_post_location = db:prepare([[
  SELECT t.board, p.thread
    FROM post_cache AS p JOIN thread_cache AS t ON p.thread = t.thread
    WHERE p.post = ?
]])

local get_thread_length = db:prepare([[
  SELECT COUNT() FROM posts WHERE thread = ?;
]])

local get_thread_staleness = db:prepare([[
  SELECT STRFTIME('%s', 'now') - MAX(date) FROM posts WHERE thread = ?;
]])

local get_excerpt = db:prepare([[
  SELECT IIF(subject <> '', subject, IIF(comment <> '', comment, name))
    FROM posts AS p LEFT JOIN file_refs AS r ON p.post = r.post
    WHERE p.post = ? LIMIT 1
]])

local get_log_count = db:prepare([[
  SELECT COUNT() FROM (SELECT 1 FROM mod_changes GROUP BY action, table_id, row_key)
]])

local has_mod_ids = db:prepare([[
  SELECT COUNT() FROM boards WHERE board = ? AND mod_ids = 1
]])

local has_new_threads_disabled = db:prepare([[
  SELECT COUNT() FROM boards WHERE board = ? AND new_threads_disabled = 1
]])

local is_text_only = db:prepare([[
  SELECT COUNT() FROM boards WHERE board = ? AND text_only = 1
]])

local boardlist = ''
for _, board in ipairs(config.LISTED_BOARDS) do
   boardlist = boardlist .. ' <a href="/' .. board .. '/?boardlinkview{{view}}">/' .. board .. '/</a>'
end
boardlist = template(boardlist)

local function is_i2p()
  return fcgi.getenv('HTTP_HOST'):find('%.i2p:?%d*$') and true or false
end

local function get_theme()
  local host = fcgi.getenv('HTTP_HOST')
  return host:find(is_i2p() and ':8080$' or '^dark%.') and 'dark' or 'light'
end

local navlinks = template(
  common.read_all('navlinks.html'), {
    boardlist = boardlist,
    toggletheme = function()
      local host = fcgi.getenv('HTTP_HOST')
      if is_i2p() then
        host = (host .. ':8080'):gsub(':8080:8080$', ''):gsub(':%d+:8080$', ':8080')
      else
        host = ('dark.' .. host):gsub('^dark%.dark%.', '')
      end
      return gen.escape('//' .. host .. fcgi.getenv('REQUEST_URI'))
    end,
    opptheme = function()
      return (get_theme() == 'dark') and 'light' or 'dark'
    end
  }
)

local function mtime(stylesheet)
  return function()
    return sys_stat.stat(stylesheet).st_mtime
  end
end

local function is_theme(theme)
  return function()
    return theme == get_theme()
  end
end

local head = template(
  HEADERS .. common.read_all('head.html'), {
    navlinks = navlinks,
    top = true,
    mtimestyle = mtime('style.css'),
    mtimelight = mtime('light.css'),
    mtimedark = mtime('dark.css'),
    dark = is_theme('dark'),
    light = is_theme('light')
  }
)

local movebuttons = ''
for _, board in ipairs(config.MOVE_OPTIONS) do
  movebuttons = movebuttons .. (
    '<input name="action_move_' .. board .. '" type="submit" value="&#8594;/' .. board .. '/" />'
  )
end

local tail = template(
  common.read_all('tail.html'), {
    uri = function() return fcgi.getenv('REQUEST_URI') end,
    movebuttons = movebuttons,
    navlinks = navlinks,
    maxreason = config.MAX_LENGTH.reason,
    minboard = config.MIN_LENGTH.board,
    maxboard = config.MAX_LENGTH.board,
    maxsubject = config.MAX_LENGTH.subject,
    maxcomment = config.MAX_LENGTH.comment,
    minop = config.MIN_LENGTH.op,
    captchawidth = config.MAX_LENGTH.captcha * 40,
    maxcaptcha = config.MAX_LENGTH.captcha
  }
)

local guide = common.read_all('guide.html')

local captcha_page = template(
  HEADERS .. common.read_all('captcha.html')
)

local error_page = template(
  '{head()}<h1 class="error">{title}</h1>\n</body></html>\n', {
    head = head,
    view = 'error'
  }
)

local function page_list(first, base, current, count)
  if count <= 1 then
    return nil
  end

  local html = 'page'
  local last_page = 0

  local function link(page)
    return (page == 1) and first or (base .. page)
  end

  local function add_page(page)
    if page <= last_page or page < 1 or page > count then
      return
    end
    if page > last_page + 1 then
      html = html .. ' ...'
    end
    local class = (page == current) and ' class="current"' or ''
    html = html .. ' <a' .. class .. ' href="' .. link(page) .. '">' .. page .. '</a>'
    last_page = page
  end

  if current > 1 then
    html = html .. ' <a rel="prev" href="' .. link(current - 1) .. '">&lt;&lt;</a>'
  end

  for i = 1, 1 + config.PAGES_START do
    add_page(i)
  end

  local j = current
  j = math.min(j, count - config.PAGES_END - 1 - config.PAGES_CURRENT)
  j = math.max(j, 1 + config.PAGES_START + 1 + config.PAGES_CURRENT)
  for i = j - config.PAGES_CURRENT, j + config.PAGES_CURRENT do
    add_page(i)
  end

  for i = count - config.PAGES_END, count do
    add_page(i)
  end

  if current < count then
    html = html .. ' <a rel="next" href="' .. link(current + 1) .. '">&gt;&gt;</a>'
  end

  return html
end

local function redirect(write, location)
  write('Status: 302 Found\r\nLocation: ' .. location .. '\r\n\r\n')
end

local function forbidden(write)
  write('Status: 403 Forbidden\r\n' .. error_page{title = '403 Forbidden'})
end

local function notfound(write)
  write('Status: 404 Not Found\r\n' .. error_page{title = '404 Not Found'})
end

local function find_post(write, _, post)
  post = tonumber(post)
  local board, thread = get_post_location(post)
  if board then
    return redirect(write, '/' .. board .. '/' .. thread .. '#P' .. post)
  else
    return notfound(write)
  end
end

local function serve_thread(write, uri, board, thread)
  thread = tonumber(thread)
  local correct_board = get_thread_location(thread)
  if not correct_board then
    local new_board, new_thread = get_post_location(thread)
    if new_board then
      return redirect(write, '/' .. new_board .. '/' .. new_thread)
    else
      return notfound(write)
    end
  end
  local expected = '/' .. correct_board .. '/' .. thread
  if uri ~= expected then
    return redirect(write, expected)
  end
  local title = gen.excerpt(get_excerpt(thread) or 'Thread ' .. thread)
  local lock_notice = ''
  if get_thread_length(thread) >= config.THREAD_MAX_POSTS then
    lock_notice = '<div class="notice">This thread has reached '
      .. config.THREAD_MAX_POSTS .. ' posts and is locked.</div>'
  end
  local is_necro = (get_thread_staleness(thread) or 0) > 30*86400
  local subs = {
    view = 'thread', board = board, thread = thread, title = title,
    posting = (lock_notice == ''), parent = thread,
    captchamode = (is_necro and 'thread' or 'reply'),
    files = (is_text_only(board) ~= 1), nomodids = (has_mod_ids(board) ~= 1),
    postbutton = 'Post Reply'
  }
  local html = get_thread(thread)
  write(head(subs))
  write(html)
  write(lock_notice)
  write(tail(subs))
end

local function serve_board(write, uri, board, view, page)
  local overboard = overboard_type(board)
  view = view or 'index'
  page = math.max(tonumber(page) or 1, 1)
  if view == 'recent' then
    page = math.min(page, config.MAX_RECENT_PAGES)
  end
  local expected =
    '/' .. board .. '/'
      .. ((view == 'index' and page == 1) and '' or view)
      .. (page == 1 and '' or page)
  if uri ~= expected then
    return redirect(write, expected)
  end
  local count = (view == 'recent' and get_recent_post_count or get_thread_count)[overboard]{board = board}
  if view == 'recent' then
    count = math.min(count, config.MAX_RECENT_PAGES * config.ITEMS_PER_PAGE.recent)
  end
  if
    not overboard and count == 0 and (
      board:len() < config.MIN_LENGTH.board or board:len() >= config.MAX_LENGTH.board
    )
  then
    return notfound(write)
  end
  local html = get_board[overboard][view]{
    board = board,
    limit = config.ITEMS_PER_PAGE[view],
    offset = (page - 1) * config.ITEMS_PER_PAGE[view]
  }
  local page_count = math.ceil(count / config.ITEMS_PER_PAGE[view])
  local first_page = (view == 'index') and '.' or view
  local pages = page_list(first_page, view, page, page_count)
  local lock_notice = ''
  if not overboard and has_new_threads_disabled(board) == 1 then
    lock_notice = '<div class="notice">New threads have been disabled on this board.</div>'
  end
  local subs = {
    view = view, board = board, title = view,
    boardlinkview = (view ~= 'index'), pages = pages,
    posting = (lock_notice == ''), parent = (not overboard and board or nil), overboard = overboard,
    captchamode = 'thread', files = (is_text_only(board) ~= 1), nomodids = (has_mod_ids(board) ~= 1),
    postbutton = 'New Thread', newthread = true
  }
  write(head(subs))
  write(html)
  write(lock_notice)
  write(tail(subs))
end

local function serve_latest(write, _, board)
  local overboard = overboard_type(board)
  local latest = get_latest[overboard]{board = board}
  write('Content-Type: text/plain; charset=utf-8\r\n\r\n' .. latest)
end

local function serve_log(write, uri, page)
  page = math.max(tonumber(page) or 1, 1)
  local expected = '/Log/' .. (page == 1 and '' or page)
  if uri ~= expected then
    return redirect(write, expected)
  end
  local count = get_log_count()
  local html = get_log(
    config.ITEMS_PER_PAGE.log, (page - 1) * config.ITEMS_PER_PAGE.log
  )
  local page_count = math.ceil(count / config.ITEMS_PER_PAGE.log)
  local pages = page_list('.', '', page, page_count)
  local subs = {view = 'log', title = 'Moderation Log', pages = pages}
  write(head(subs))
  write(html)
  write(tail(subs))
end

local function serve_guide(write, uri)
  local subs = {view = 'guide', title = 'Guide to Lambdaplusjs'}
  local expected = '/Guide/'
  if uri ~= expected then
    return redirect(write, expected)
  end
  write(head(subs))
  write(guide)
  write(tail(subs))
end

local function serve_captcha(write, _, post_type)
  local block_time, captcha_time = captcha.cooldown(post_type == 'thread')
  local msg, img
  if block_time > 0 then
    msg = 'wait ' .. block_time .. 's to post'
  elseif captcha_time > 0 then
    img = captcha.get_one()
  else
    msg = 'not needed'
  end
  return write(captcha_page{
    captchamessage = msg,
    captchaimage = img
  })
end

local route_rules = {
  {'^/(%l+)/(%d+)/?$',            serve_thread},
  {'^/(%l+)/?$',                  serve_board},
  {'^/(%l+)/(index)(%d*)/?$',     serve_board},
  {'^/(%l+)/(catalog)(%d*)/?$',   serve_board},
  {'^/(%l+)/(recent)(%d*)/?$',    serve_board},
  {'^/(%l+)/latest/?$',           serve_latest},
  {'^/(All2?)/?$',                serve_board},
  {'^/(All2?)/(index)(%d*)/?$',   serve_board},
  {'^/(All2?)/(catalog)(%d*)/?$', serve_board},
  {'^/(All2?)/(recent)(%d*)/?$',  serve_board},
  {'^/(All2?)/latest/?$',         serve_latest},
  {'^/Log/?(%d*)/?$',             serve_log},
  {'^/Guide/?$',                  serve_guide},
  {'^/Captcha/(thread)/?$',       serve_captcha},
  {'^/Captcha/(reply)/?$',        serve_captcha},
  {'^/P(%d+)/?$',                 find_post}
}

local function route(write)
  if fcgi.getenv('REDIRECT_STATUS') == '403' then
    return forbidden(write)
  end
  local uri = fcgi.getenv('REQUEST_URI')
  for _, rule in ipairs(route_rules) do
    local match = {uri:find(rule[1])}
    if match[1] then
      return rule[2](write, uri, unpack(match, 3))
    end
  end
  return notfound(write)
end

while fcgi.accept() do
  common.handle_errors(route, fcgi.print)
end
