#include "sqlite_database.h"

#include <utility>

#include "sqlite_prepared_statement.h"

namespace database {

SqliteDatabase::SqliteDatabase(const char* filename) {
  if (sqlite3_open(filename, &handle) != SQLITE_OK) {
    throw std::runtime_error(sqlite3_errmsg(handle));
  }
  sqlite3_busy_timeout(handle, 1000);
}

SqliteDatabase::~SqliteDatabase() { sqlite3_close(handle); }

SqliteDatabase::SqliteDatabase(SqliteDatabase&& other) noexcept { handle = std::exchange(other.handle, nullptr); }

SqliteDatabase& SqliteDatabase::operator=(SqliteDatabase&& other) noexcept {
  std::swap(handle, other.handle);
  return *this;
}

std::unique_ptr<PreparedStatement> SqliteDatabase::prepareStatement(std::string_view sql) {
  return std::make_unique<SqlitePreparedStatement>(handle, sql);
}

void SqliteDatabase::transactionBegin() {
  if (sqlite3_exec(handle, "BEGIN", nullptr, nullptr, nullptr) != SQLITE_OK) {
    throw std::runtime_error(sqlite3_errmsg(handle));
  }
}

void SqliteDatabase::transactionEnd() {
  if (sqlite3_exec(handle, "END", nullptr, nullptr, nullptr) != SQLITE_OK) {
    throw std::runtime_error(sqlite3_errmsg(handle));
  }
}

void SqliteDatabase::transactionRollback() {
  if (sqlite3_exec(handle, "ROLLBACK", nullptr, nullptr, nullptr) != SQLITE_OK) {
    throw std::runtime_error(sqlite3_errmsg(handle));
  }
}

}  // namespace database