#ifndef NANOCHAN_DATABASE_SQLITE_SQLITE_PREPARED_STATEMENT_H
#define NANOCHAN_DATABASE_SQLITE_SQLITE_PREPARED_STATEMENT_H

#include <sqlite3.h>

#include <string_view>

#include "../prepared_statement.h"

namespace database {

class SqlitePreparedStatement : public PreparedStatement {
 private:
  sqlite3* dbHandle;
  sqlite3_stmt* handle = nullptr;

 public:
  SqlitePreparedStatement(sqlite3* dbHandle, std::string_view sql);
  ~SqlitePreparedStatement() override;
  SqlitePreparedStatement(const SqlitePreparedStatement& other) = delete;
  SqlitePreparedStatement& operator=(const SqlitePreparedStatement& other) = delete;
  SqlitePreparedStatement(SqlitePreparedStatement&& other) noexcept;
  SqlitePreparedStatement& operator=(SqlitePreparedStatement&& other) noexcept;
  void bind(int column, bool data) override;
  void bind(int column, int64_t data) override;
  void bind(int column, std::string_view data) override;
  std::span<const uint8_t> getBlob(int column) override;
  bool getBool(int column) override;
  int64_t getInt(int column) override;
  std::string getString(int column) override;
  QueryResult step() override;
};

}  // namespace database

#endif  // NANOCHAN_DATABASE_SQLITE_SQLITE_PREPARED_STATEMENT_H
