#include "font_face.h"

#include <stdexcept>
#include <utility>

namespace font {

FontFace::FontFace(FT_Face face) : handle(face) {}

FontFace::~FontFace() { FT_Done_Face(handle); }

FontFace::FontFace(FontFace &&other) noexcept { handle = std::exchange(other.handle, nullptr); }

FontFace &FontFace::operator=(FontFace &&other) noexcept {
  std::swap(handle, other.handle);
  return *this;
}

void FontFace::setSize(unsigned int height) {
  if (FT_Set_Pixel_Sizes(handle, 0, height)) {
    throw std::runtime_error("Could not set font size");
  }
}

FontGlyph FontFace::rasterize(char c) {
  int glyphIndex = FT_Get_Char_Index(handle, c);
  if (FT_Load_Glyph(handle, glyphIndex, FT_LOAD_DEFAULT)) {
    throw std::runtime_error("Could not load glyph");
  }
  FT_Glyph glyph;
  if (FT_Get_Glyph(handle->glyph, &glyph)) {
    throw std::runtime_error("Could not get glyph");
  }
  if (FT_Glyph_To_Bitmap(&glyph, FT_RENDER_MODE_NORMAL, nullptr, 1)) {
    throw std::runtime_error("Could not convert glyph to bitmap");
  }
  auto bitmapGlyph = reinterpret_cast<FT_BitmapGlyph>(glyph);
  FT_Bitmap bitmap = bitmapGlyph->bitmap;
  for (unsigned int y = 0; y < bitmap.rows; y++) {
    for (unsigned int x = 0; x < bitmap.width; x++) {
      bitmap.buffer[y * bitmap.width + x] = 0xFF - bitmap.buffer[y * bitmap.width + x];
    }
  }
  return FontGlyph(bitmapGlyph);
}

}  // namespace font