#include "freetype.h"

#include <stdexcept>
#include <utility>

#include "font_face.h"
#include "font_pattern.h"
#include "name_pattern.h"

namespace font {

Freetype::Freetype() {
  if (FT_Init_FreeType(&handle)) {
    throw std::runtime_error("Could not initialize freetype");
  }
}

Freetype::~Freetype() {
  FT_Done_FreeType(handle);
}

Freetype::Freetype(Freetype &&other) noexcept {
  handle = std::exchange(other.handle, nullptr);
}

Freetype &Freetype::operator=(Freetype &&other) noexcept {
  std::swap(handle, other.handle);
  return *this;
}

FontFace Freetype::open(std::string_view name) {
  font::NamePattern namePattern(fontConfig, name);
  font::FontPattern fontPattern(fontConfig, namePattern);
  char *path;
  fontPattern.path(&path);
  FT_Face face;
  if (FT_New_Face(handle, path, 0, &face)) {
    throw std::runtime_error("Could not open font");
  }
  return FontFace(face);
}

}  // namespace font