#include "format_context.h"

#include <stdexcept>
#include <utility>

namespace media {

FormatContext::FormatContext(const char *filename, IOContext&& _ioContext) : ioContext(std::move(_ioContext)) {
  context = avformat_alloc_context();
  if (!context) {
    throw std::runtime_error("Could not allocate format context");
  }
  context->pb = ioContext.context;
  if (avformat_open_input(&context, filename, nullptr, nullptr) < 0) {
    throw std::runtime_error("Could not open input stream");
  }
}

FormatContext::~FormatContext() { avformat_close_input(&context); }

FormatContext::FormatContext(FormatContext&& other) noexcept : ioContext(std::move(other.ioContext)) {
  context = std::exchange(other.context, nullptr);
}

FormatContext& FormatContext::operator=(FormatContext&& other) noexcept {
  std::swap(context, other.context);
  ioContext = std::move(other.ioContext);
  return *this;
}

bool FormatContext::readFrame(const Packet& packet) const {
  av_packet_unref(packet.handle);
  return !av_read_frame(context, packet.handle);
}

int64_t FormatContext::frameTimestamp(const Packet& packet, const Frame& frame) const {
  AVRational time_base = context->streams[packet.handle->stream_index]->time_base;
  return frame.handle->best_effort_timestamp * time_base.num / time_base.den;
}

}  // namespace media
