#include "image.h"

#include <cstdint>
extern "C" {
#include <libavutil/imgutils.h>
}
#include <stdexcept>
#include <utility>

namespace media {

Image::Image(int width, int height) {
  if (av_image_alloc(planes, strides, width, height, AV_PIX_FMT_RGBA, 16) < 0) {
    throw std::runtime_error("Could not allocate image");
  }
}

Image::~Image() { av_freep(&planes[0]); }

Image::Image(Image&& other) noexcept {
  for (size_t i = 0; i < 4; i++) {
    planes[i] = std::exchange(other.planes[i], nullptr);
    strides[i] = other.strides[i];
  }
}

Image& Image::operator=(Image&& other) noexcept {
  for (size_t i = 0; i < 4; i++) {
    std::swap(planes[i], other.planes[i]);
    std::swap(strides[i], other.strides[i]);
  }
  return *this;
}

}  // namespace media
