#include "jpeg_compressor.h"

#include <stdexcept>
#include <utility>

namespace media {

JPEGCompressor::JPEGCompressor() {
  handle = tjInitCompress();
  if (!handle) {
    throw std::runtime_error("Could not initialize turbojpeg");
  }
}

JPEGCompressor::~JPEGCompressor() { tjDestroy(handle); }

JPEGCompressor::JPEGCompressor(JPEGCompressor&& other) noexcept { handle = std::exchange(other.handle, nullptr); }

JPEGCompressor& JPEGCompressor::operator=(JPEGCompressor&& other) noexcept {
  std::swap(handle, other.handle);
  return *this;
}

JPEG JPEGCompressor::compress(const unsigned char* data, const int width, const int stride, const int height,
                              const int quality) const {
  return compress(data, width, stride, height, quality, false);
}

JPEG JPEGCompressor::compress(const unsigned char* data, const int width, const int stride, const int height,
                              const int quality, const bool grayscale) const {
  uint8_t* imageBuffer = nullptr;
  unsigned long imageSize = 0;
  if (tjCompress2(handle, data, width, stride, height, grayscale ? TJPF_GRAY : TJPF_RGB, &imageBuffer, &imageSize,
                  grayscale ? TJSAMP_GRAY : TJSAMP_444, quality, TJFLAG_FASTDCT) == -1) {
    throw std::runtime_error("Could not compress image");
  }
  return JPEG(imageBuffer, imageSize, width, height);
}

}  // namespace media
