#include "pdf_thumbnailer.h"

#include <poppler/cpp/poppler-document.h>

#include <iostream>

#include "jpeg_compressor.h"

namespace media {

PDFThumbnailer::PDFThumbnailer(const std::string_view contents) {
  document = poppler::document::load_from_raw_data(contents.data(), contents.length());
  if (!document) {
    throw std::runtime_error("Could not load document");
  }
  page = document->create_page(0);
  if (!page) {
    delete document;
    throw std::runtime_error("Could not create page");
  }
  renderer.set_image_format(poppler::image::format_enum::format_rgb24);
}

PDFThumbnailer::~PDFThumbnailer() {
  delete page;
  delete document;
}

std::optional<JPEG> PDFThumbnailer::scale(const int maxWidth, const int maxHeight) {
  int width = page->page_rect().width();
  int height = page->page_rect().height();
  if (width > maxWidth || height > maxHeight) {
    double ratioX = page->page_rect().width() / maxWidth;
    double ratioY = page->page_rect().height() / maxHeight;
    if (ratioX >= ratioY) {
      width = maxWidth;
      height /= ratioX;
    } else {
      width /= ratioY;
      height = maxHeight;
    }
  }
  poppler::image image = renderer.render_page(page, 72.0 / (page->page_rect().width() / width),
                                              72.0 / (page->page_rect().height() / height), -1, -1, width, height);
  if (!image.is_valid()) {
    return std::nullopt;
  }
  JPEGCompressor compressor;
  return compressor.compress(reinterpret_cast<unsigned char *>(image.data()), image.width(), image.bytes_per_row(),
                             image.height(), 60);
}

std::vector<uint8_t> PDFThumbnailer::dhash() {
  std::vector<uint8_t> hash;
  hash.reserve(9 * 8);
  poppler::image image = renderer.render_page(page, 72.0 / (page->page_rect().width() / 9),
                                              72.0 / (page->page_rect().height() / 8), -1, -1, 9, 8);
  if (!image.is_valid()) {
    return hash;
  }
  for (int y = 0; y < 8; y++) {
    for (int x = 0; x < 8; x++) {
      uint8_t p1 = image.data()[y * image.bytes_per_row() + x];
      uint8_t p2 = image.data()[y * image.bytes_per_row() + x + 1];
      hash.push_back(p2 - p1);
    }
  }
  return hash;
}

}  // namespace media
