#include "thumbnailer_factory.h"

#include <iostream>

#include "../util/util.h"
#include "pdf_thumbnailer.h"
#include "video_thumbnailer.h"

namespace media {

bool ThumbnailerFactory::hasThumbnail(const std::string_view extension) {
  util::FileClass fileClass = util::fileClass(extension);
  return fileClass == util::FileClass::image || fileClass == util::FileClass::pdf || fileClass == util::FileClass::video;
}

std::unique_ptr<Thumbnailer> ThumbnailerFactory::create(const std::string& filename,
                                                        const std::string_view contents) {
  util::FileClass fileClass = util::fileClass(util::extension(filename));
  try {
    switch (fileClass) {
      case util::FileClass::image:
        return std::make_unique<VideoThumbnailer>(filename.c_str(), contents, true);
      case util::FileClass::pdf:
        return std::make_unique<PDFThumbnailer>(contents);
      case util::FileClass::video:
        return std::make_unique<VideoThumbnailer>(filename.c_str(), contents, false);
      default:
        return nullptr;
    }
  }
  catch (const std::runtime_error& exception)
  {
    std::cerr << "Failed to create thumbnailer: " << exception.what() << std::endl;
    return nullptr;
  }
}

}  // namespace media
