#ifndef NANOCHAN_CATALOG_H
#define NANOCHAN_CATALOG_H

#include <string>
#include <vector>

#include "../imageboard/imageboard.h"
#include "../util/util.h"

namespace nanochan {

/**
 * @brief Catalog page template
 */
class Catalog {
 private:
  /**
   * @brief Site wide announcement
   */
  const std::string_view announcement;
  /**
   * @brief A list of boards
   */
  const std::vector<imageboard::Board>& boards;
  /**
   * @brief A banner image for the board
   */
  const std::string_view banner;
  /**
   * @brief The board the catalog is for
   */
  const imageboard::Board& board;
  /**
   * @brief A list of threads
   */
  const std::vector<imageboard::Thread>& threads;
  /**
   * @brief Whether ThreadCaptcha is enabled
   */
  const bool threadCaptcha;

 public:
  /**
   * @brief Constructs the template
   *
   * @param announcement The announcement to use
   * @param boards The boards to use
   * @param board The board this is for
   * @param threads The threads to use
   * @param threadCaptcha Whether ThreadCaptcha is enabled
   */
  Catalog(const std::string_view& announcement, const std::vector<imageboard::Board>& boards,
          const std::string_view banner, const imageboard::Board& board, const std::vector<imageboard::Thread>& threads,
          const bool threadCaptcha)
      : announcement(announcement),
        boards(boards),
        banner(banner),
        board(board),
        threads(threads),
        threadCaptcha(threadCaptcha){};
  /**
   * @brief Renders the template
   *
   * @return The rendered template
   */
  std::string render();
};

}  // namespace nanochan

#endif
