#ifndef NANOCHAN_ERROR_H
#define NANOCHAN_ERROR_H

#include <string>
#include <vector>

#include "../imageboard/imageboard.h"

namespace nanochan {

/**
 * @brief Error page template
 */
class Error {
 private:
  /**
   * @brief A list of boards
   */
  const std::vector<imageboard::Board>& boards;
  /**
   * @brief The reason the server had an error
   */
  const std::string_view reason;
  /**
   * @brief A description for what to do
   */
  const std::string_view description;

 public:
  /**
   * @brief Constructs the template
   *
   * @param boards A list of boards
   * @param reason The reason the server had an error
   * @param description A description for what to do
   */
  Error(const std::vector<imageboard::Board>& boards, const std::string_view reason, const std::string_view description)
      : boards(boards), reason(reason), description(description){};
  /**
   * @brief Renders the template
   *
   * @return The rendered template
   */
  std::string render();
};

}  // namespace nanochan

#endif
