#ifndef NANOCHAN_LOG_H
#define NANOCHAN_LOG_H

#include <string>
#include <vector>

#include "../imageboard/imageboard.h"

namespace nanochan {

/**
 * @brief Log page template
 */
class Log {
 private:
  /**
   * @brief A list of boards
   */
  const std::vector<imageboard::Board>& boards;
  /**
   * @brief A list of log entries
   */
  const std::vector<imageboard::LogEntry>& entries;
  /**
   * @brief The current page of logs
   */
  const int page;

 public:
  /**
   * @brief Constructs the template
   *
   * @param boards The boards to use
   * @param entries The log entries to use
   * @param page The page to use
   */
  Log(const std::vector<imageboard::Board>& boards, const std::vector<imageboard::LogEntry>& entries, const int page)
      : boards(boards), entries(entries), page(page){};
  /**
   * @brief Renders the template
   *
   * @return The rendered template
   */
  std::string render();
};

}  // namespace nanochan

#endif
