#ifndef NANOCHAN_OVERBOARD_H
#define NANOCHAN_OVERBOARD_H

#include <string>
#include <vector>

#include "../imageboard/imageboard.h"
#include "../util/util.h"

namespace nanochan {

/**
 * @brief Overboard page template
 */
class Overboard {
 private:
  /**
   * @brief Site wide announcement
   */
  const std::string_view announcement;
  /**
   * @brief A list of boards
   */
  const std::vector<imageboard::Board>& boards;
  /**
   * @brief A list of threads
   */
  const std::vector<imageboard::Thread>& threads;

 public:
  /**
   * @brief Constructs the template
   *
   * @param announcement The announcement to use
   * @param boards The boards to use
   * @param threads The threads to use
   */
  Overboard(const std::string_view announcement, const std::vector<imageboard::Board>& boards,
            const std::vector<imageboard::Thread>& threads)
      : announcement(announcement), boards(boards), threads(threads){};
  /**
   * @brief Renders the template
   *
   * @return The rendered template
   */
  std::string render();
};

}  // namespace nanochan

#endif
