#ifndef NANOCHAN_RECENT_H
#define NANOCHAN_RECENT_H

#include <string>
#include <vector>

#include "../imageboard/imageboard.h"
#include "../util/util.h"

namespace nanochan {

/**
 * @brief Recent page template
 */
class Recent {
 private:
  /**
   * @brief A list of boards
   */
  const std::vector<imageboard::Board>& boards;
  /**
   * @brief A list of posts
   */
  const std::vector<imageboard::Post>& posts;
  /**
   * @brief The current page of posts
   */
  const int page;
  /**
   * @brief Whether only posts which sage should be shown
   */
  const bool sage = true;
  /**
   * @brief Whether only posts with files should be shown
   */
  const bool file = true;
  /**
   * @brief Whether only original posts should be shown
   */
  const bool parent = true;
  /**
   * @brief Whether only posts with a custom name should be shown
   */
  const bool custName = true;
  /**
   * @brief Whether only posts deleted by tvols should be shown
   */
  const bool tvolDelete = true;
  /**
   * @brief The number of posts per page
   */
  const std::vector<imageboard::Post>::size_type postLimit;
  /**
   * @brief Should posts start checked
   */
  const bool checkedPosts;
  /**
   * @brief Whether mod controls should be rendered
   */
  const bool mod = true;
  /**
   * @brief Show board names on posts
   */
  const bool postBoardName = true;

 public:
  /**
   * @brief Constructs the template
   *
   * @param boards The boards to use
   * @param posts The posts to use
   * @param page The page to use
   * @param checkedPosts Should posts start checked
   */
  Recent(const std::vector<imageboard::Board>& boards, const std::vector<imageboard::Post>& posts, const int page,
         const bool sage, const bool file, const bool parent, const bool custName, const bool tvolDelete,
         const int postLimit, const bool checkedPosts)
      : boards(boards),
        posts(posts),
        page(page),
        sage(sage),
        file(file),
        parent(parent),
        custName(custName),
        tvolDelete(tvolDelete),
        postLimit(postLimit),
        checkedPosts(checkedPosts){};
  /**
   * @brief Renders the template
   *
   * @return The rendered template
   */
  std::string render();
};

}  // namespace nanochan

#endif
