#ifndef NANOCHAN_THREAD_H
#define NANOCHAN_THREAD_H

#include <string>
#include <vector>

#include "../imageboard/imageboard.h"
#include "../util/util.h"

namespace nanochan {

/**
 * @brief Recent page template
 */
class Thread {
 private:
  /**
   * @brief Site wide announcement
   */
  const std::string_view announcement;
  /**
   * @brief A list of boards
   */
  const std::vector<imageboard::Board>& boards;
  /**
   * @brief A banner image for the board
   */
  const std::string_view banner;
  /**
   * @brief The board the catalog is on
   */
  const imageboard::Board& board;
  /**
   * @brief A list of posts
   */
  const std::vector<imageboard::Post>& posts;
  /**
   * @brief A count of files in the thread
   */
  const std::vector<imageboard::Board>::size_type fileCount;
  /**
   * @brief Whether mod controls should be rendered
   */
  const bool mod = true;
  /**
   * @brief Do not show board names on posts
   */
  const bool postBoardName = false;

 public:
  /**
   * @brief Constructs the template
   *
   * @param announcement The announcement to use
   * @param boards The boards to use
   * @param board The board the catalog is on
   * @param posts The posts to use
   * @param fileCount A count of files in the thread
   */
  Thread(const std::string_view announcement, const std::vector<imageboard::Board>& boards,
         const std::string_view banner, const imageboard::Board& board, const std::vector<imageboard::Post>& posts,
         const std::vector<imageboard::Board>::size_type fileCount)
      : announcement(announcement), boards(boards), banner(banner), board(board), posts(posts), fileCount(fileCount){};
  /**
   * @brief Renders the template
   *
   * @return The rendered template
   */
  std::string render();
};

}  // namespace nanochan

#endif
