#ifndef SCGI_PARSER_H
#define SCGI_PARSER_H

#include <string_view>
#include <unordered_map>

namespace scgi {

/**
 * @brief Handles parsing SCGI requests
 */
class Parser {
 public:
  /**
   * @brief Decodes multipart encoded content
   *
   * @param contents The contents to parse
   * @param boundary The boundary seperating parts
   * @return A map of part names to part contents
   */
  static std::pair<std::unordered_map<std::string_view, std::string_view>,
                   std::unordered_map<std::string_view, std::string_view>>
  parseMultiPart(const std::string_view contents,
                 const std::string_view boundary);
  /**
   * @brief Parses a content disposition header
   *
   * @param contentDisposition The header to extract from
   * @return A map of key value pairs
   */
  static std::unordered_map<std::string_view, std::string_view>
  parseContentDisposition(const std::string_view contentDisposition);
  /**
   * @brief Decodes URL encoded parameters
   *
   * If there is an error parsing the parameters then parsing will
   * stop.
   * You can pass in -1 for the length and parsing will stop once a
   * null terminator is reached.
   * @param queryParams A URL encoded string
   * @return A map of key value pairs
   */
  static std::unordered_map<std::string_view, std::string_view> urlDecode(
      const std::string_view& queryParams);
};

}  // namespace scgi

#endif
