#include "parser.h"

#include <gtest/gtest.h>

TEST(scgiParser, parseMultipart) {
  std::unordered_map<std::string_view, std::string_view> zero;
  std::unordered_map<std::string_view, std::string_view> oneE = {{"a", ""}};
  std::unordered_map<std::string_view, std::string_view> one = {{"a", "b"}};
  std::unordered_map<std::string_view, std::string_view> oneEFile = {{"c", ""}};
  std::unordered_map<std::string_view, std::string_view> oneFile = {{"c", "e"}};
  std::unordered_map<std::string_view, std::string_view> twoE = {{"a", "b"},
                                                                 {"c", ""}};
  std::unordered_map<std::string_view, std::string_view> two = {{"a", "b"},
                                                                {"c", "d"}};
  EXPECT_EQ(
      scgi::Parser::parseMultiPart("", "---------------------------boundary"),
      std::make_pair(zero, zero));
  EXPECT_EQ(scgi::Parser::parseMultiPart("test",
                                         "---------------------------boundary"),
            std::make_pair(zero, zero));
  EXPECT_EQ(
      scgi::Parser::parseMultiPart("-----------------------------boundary",
                                   "---------------------------boundary"),
      std::make_pair(zero, zero));
  EXPECT_EQ(
      scgi::Parser::parseMultiPart("-----------------------------boundary\r\n",
                                   "---------------------------boundary"),
      std::make_pair(zero, zero));
  EXPECT_EQ(
      scgi::Parser::parseMultiPart("-----------------------------boundary\r\n"
                                   "Content-Disposition: form-data; name=\"a\"",
                                   "---------------------------boundary"),
      std::make_pair(zero, zero));
  EXPECT_EQ(scgi::Parser::parseMultiPart(
                "-----------------------------boundary\r\n"
                "Content-Disposition: form-data; name=\"a\"\r\n",
                "---------------------------boundary"),
            std::make_pair(zero, zero));
  EXPECT_EQ(scgi::Parser::parseMultiPart(
                "-----------------------------boundary\r\n"
                "Content-Disposition: form-data; name=\"a\"\r\n"
                "\r\n",
                "---------------------------boundary"),
            std::make_pair(oneE, zero));
  EXPECT_EQ(scgi::Parser::parseMultiPart(
                "-----------------------------boundary\r\n"
                "Content-Disposition: form-data; name=\"a\"\r\n"
                "\r\n"
                "b",
                "---------------------------boundary"),
            std::make_pair(one, zero));
  EXPECT_EQ(scgi::Parser::parseMultiPart(
                "-----------------------------boundary\r\n"
                "Content-Disposition: form-data; name=\"a\"\r\n"
                "\r\n"
                "b\r\n"
                "-----------------------------boundary",
                "---------------------------boundary"),
            std::make_pair(one, zero));
  EXPECT_EQ(scgi::Parser::parseMultiPart(
                "-----------------------------boundary\r\n"
                "Content-Disposition: form-data; name=\"a\"\r\n"
                "\r\n"
                "b\r\n"
                "-----------------------------boundary--",
                "---------------------------boundary"),
            std::make_pair(one, zero));
  EXPECT_EQ(scgi::Parser::parseMultiPart(
                "-----------------------------boundary\r\n"
                "Content-Disposition: form-data; name=\"a\"\r\n"
                "\r\n"
                "b\r\n"
                "-----------------------------boundary\r\n"
                "Content-Disposition: form-data; name=\"c\"; filename=\"\r\n"
                "Content-Type: text/plain\r\n"
                "\r\n"
                "d\r\n"
                "-----------------------------boundary--\r\n",
                "---------------------------boundary"),
            std::make_pair(two, zero));
  EXPECT_EQ(scgi::Parser::parseMultiPart(
                "-----------------------------boundary\r\n"
                "Content-Disposition: form-data; name=\"a\"\r\n"
                "\r\n"
                "b\r\n"
                "-----------------------------boundary\r\n"
                "Content-Disposition: form-data; name=\"c\"; filename=\"\"\r\n"
                "Content-Type: text/plain\r\n"
                "\r\n"
                "d\r\n"
                "-----------------------------boundary--\r\n",
                "---------------------------boundary"),
            std::make_pair(two, oneEFile));
  EXPECT_EQ(scgi::Parser::parseMultiPart(
                "-----------------------------boundary\r\n"
                "Content-Disposition: form-data; name=\"a\"\r\n"
                "\r\n"
                "b\r\n"
                "-----------------------------boundary\r\n"
                "Content-Disposition: form-data; name=\"c\"; filename=\"e\"\r\n"
                "Content-Type: text/plain\r\n"
                "\r\n"
                "d\r\n"
                "-----------------------------boundary--\r\n",
                "---------------------------boundary"),
            std::make_pair(two, oneFile));
  EXPECT_EQ(
      scgi::Parser::parseMultiPart(
          "-----------------------------boundary\r\n"
          "Content-Disposition  :  form-data; name=\"a\"  \r\n"
          "\r\n"
          "b\r\n"
          "-----------------------------boundary\r\n"
          "Content-Disposition  :  form-data; name=\"c\"; filename=\"\"  \r\n"
          "Content-Type  :  text/plain\r\n"
          "\r\n"
          "d\r\n"
          "-----------------------------boundary--\r\n",
          "---------------------------boundary"),
      std::make_pair(two, oneEFile));
  EXPECT_EQ(
      scgi::Parser::parseMultiPart(
          "-----------------------------boundary\r\n"
          "Content-Disposition  :  form-data; name=\"a\"  \r\n"
          "\r\n"
          "b\r\n"
          "-----------------------------boundary\r\n"
          "Content-Disposition  :  form-data; name=\"c\"; filename=\"e\"  \r\n"
          "Content-Type  :  text/plain\r\n"
          "\r\n"
          "d\r\n"
          "-----------------------------boundary--\r\n",
          "---------------------------boundary"),
      std::make_pair(two, oneFile));
  EXPECT_EQ(scgi::Parser::parseMultiPart(
                "--z\r\n"
                "Content-Disposition: form-data; name=\"a\"\r\n"
                "\r\n"
                "b\r\n"
                "--z--",
                "z"),
            std::make_pair(one, zero));
  EXPECT_EQ(scgi::Parser::parseMultiPart(
                "-----------------------------boundary\r\n"
                "A\r\n"
                "\r\n"
                "b\r\n"
                "-----------------------------boundary\r\n"
                "Content-Disposition: form-data; name=\"a\"\r\n"
                "\r\n"
                "b\r\n"
                "-----------------------------boundary--\r\n",
                "---------------------------boundary"),
            std::make_pair(one, zero));
  EXPECT_EQ(scgi::Parser::parseMultiPart(
                "-----------------------------boundary\r\n"
                "A:\r\n"
                "\r\n"
                "b\r\n"
                "-----------------------------boundary\r\n"
                "Content-Disposition: form-data; name=\"a\"\r\n"
                "\r\n"
                "b\r\n"
                "-----------------------------boundary--\r\n",
                "---------------------------boundary"),
            std::make_pair(one, zero));
}

TEST(scgiParser, parseContentDisposition) {
  std::unordered_map<std::string_view, std::string_view> zero;
  std::unordered_map<std::string_view, std::string_view> oneE = {{"a", ""}};
  std::unordered_map<std::string_view, std::string_view> one = {{"a", "b"}};
  std::unordered_map<std::string_view, std::string_view> twoE = {{"a", "b"},
                                                                 {"c", ""}};
  std::unordered_map<std::string_view, std::string_view> two = {{"a", "b"},
                                                                {"c", "d"}};
  EXPECT_EQ(scgi::Parser::parseContentDisposition(""), zero);
  EXPECT_EQ(scgi::Parser::parseContentDisposition("form-data;"), zero);
  EXPECT_EQ(scgi::Parser::parseContentDisposition("form-data; "), zero);
  EXPECT_EQ(scgi::Parser::parseContentDisposition("form-data; a"), zero);
  EXPECT_EQ(scgi::Parser::parseContentDisposition("form-data; a="), zero);
  EXPECT_EQ(scgi::Parser::parseContentDisposition("form-data; a=\""), zero);
  EXPECT_EQ(scgi::Parser::parseContentDisposition("form-data; a=\"\""), oneE);
  EXPECT_EQ(scgi::Parser::parseContentDisposition("form-data; a=\"b\""), one);
  EXPECT_EQ(scgi::Parser::parseContentDisposition("form-data; a=\"b\";"), one);
  EXPECT_EQ(scgi::Parser::parseContentDisposition("form-data; a=\"b\"; "), one);
  EXPECT_EQ(scgi::Parser::parseContentDisposition("form-data; a=\"b\"; c"),
            one);
  EXPECT_EQ(scgi::Parser::parseContentDisposition("form-data; a=\"b\"; c="),
            one);
  EXPECT_EQ(scgi::Parser::parseContentDisposition("form-data; a=\"b\"; c=\""),
            one);
  EXPECT_EQ(scgi::Parser::parseContentDisposition("form-data; a=\"b\"; c=\"d"),
            one);
  EXPECT_EQ(
      scgi::Parser::parseContentDisposition("form-data; a=\"b\"; c=\"d\""),
      two);
  EXPECT_EQ(scgi::Parser::parseContentDisposition("form-data;a=\"b\";c=\"d\""),
            two);
  EXPECT_EQ(scgi::Parser::parseContentDisposition(
                "form-data;  a  =  \"b\"  ;  c  =  \"d\"  "),
            two);
  EXPECT_EQ(scgi::Parser::parseContentDisposition(
                "form-data;  a  =  \"b\"  ;  c  =  \"d\"  ;"),
            two);
  EXPECT_EQ(scgi::Parser::parseContentDisposition(
                "form-data;  a  =  \"b\"  ;  c  =  \"d\"  ;  "),
            two);
}

TEST(scgiParser, urlDecode) {
  std::unordered_map<std::string_view, std::string_view> zero;
  std::unordered_map<std::string_view, std::string_view> oneE = {{"a", ""}};
  std::unordered_map<std::string_view, std::string_view> one = {{"a", "b"}};
  std::unordered_map<std::string_view, std::string_view> twoE = {{"a", "b"},
                                                                 {"c", ""}};
  std::unordered_map<std::string_view, std::string_view> two = {{"a", "b"},
                                                                {"c", "d"}};
  EXPECT_EQ(scgi::Parser::urlDecode(""), zero);
  EXPECT_EQ(scgi::Parser::urlDecode("a"), zero);
  EXPECT_EQ(scgi::Parser::urlDecode("a="), oneE);
  EXPECT_EQ(scgi::Parser::urlDecode("a=b"), one);
  EXPECT_EQ(scgi::Parser::urlDecode("a=b&"), one);
  EXPECT_EQ(scgi::Parser::urlDecode("a=b&c"), one);
  EXPECT_EQ(scgi::Parser::urlDecode("a=b&c="), twoE);
  EXPECT_EQ(scgi::Parser::urlDecode("a=b&c=d"), two);
  EXPECT_EQ(scgi::Parser::urlDecode("a=b&c=d&"), two);
}