#ifndef TEMPLATING_COMPILER_H
#define TEMPLATING_COMPILER_H

#include <optional>
#include <string>

#include "../util/posix_filesystem.h"
#include "result.h"
#include "state.h"

namespace templating {

/**
 * @brief Implements a template compiler
 */
class Compiler {
  /**
   * @brief Generates source code for template
   *
   * @param filesystem The filesystem to write generated code
   * @param header The header file of the exiting class
   * @param namespace The name of the namespace of the existing class
   * @param class The name of the existing class
   * @param out The output source file
   * @param result The parsed template to generate code for
   */
  static void generateCode(util::Filesystem& filesystem, const std::string_view header, const std::string_view ns,
                           const std::string_view cl, const std::string& filename, Result& result);

 public:
  /**
   * @brief Compiles a template
   *
   * @param header The header file of the exiting class
   * @param ns The name of the namespace of the existing class
   * @param cl The name of the existing class
   * @param in The input template
   * @param out The output source file
   */
  static bool compile(const std::string_view header, const std::string_view ns, const std::string_view cl,
                      const std::string_view in, const std::string& out);
};

}  // namespace templating

#endif
