#include "result.h"

namespace templating {

std::vector<Token> Result::flattenTokens(const std::vector<Token>& tokens) const {
  std::vector<Token> flat;
  for (const Token& token : tokens) {
    if (token.type != TokenType::block) {
      flat.push_back(token);
      continue;
    }
    std::vector<Token> block = flattenTokens(blocks.at(token.data[0]));
    std::move(block.begin(), block.end(), std::inserter(flat, flat.end()));
  }
  return flat;
}

void Result::flatten() {
  root = flattenTokens(root);
  blocks.clear();
}

std::ostream& operator<<(std::ostream& stream, const Result& result) {
  stream << "ROOT" << std::endl;
  for (const Token& token : result.root) {
    stream << token << std::endl;
  }
  for (auto& e : result.blocks) {
    stream << "BLOCK: " << e.first << std::endl;
    for (const Token& token : e.second) {
      stream << token << std::endl;
    }
  }
  return stream << "END";
}

}  // namespace templating
