#ifndef TEMPLATING_RESULT_H
#define TEMPLATING_RESULT_H

#include <ostream>
#include <vector>

#include "../../third_party/abseil-cpp/absl/container/flat_hash_map.h"
#include "token.h"

namespace templating {

/**
 * @brief Shorthand for result blocks
 */
using Blocks = absl::flat_hash_map<std::string, std::vector<Token>>;

/**
 * @brief Result of parsing a template
 *
 * This contains a list of tokens for each block and a list
 * for a root block.
 */
class Result {
 private:
  /**
   * @brief Expands block tokens of a list of tokens
   *
   * @warning Can loop forever from mutual recursion from blocks
   * @param tokens A list of tokens to flatten
   * @return A list of tokens without block tokens
   */
  std::vector<Token> flattenTokens(const std::vector<Token>& tokens) const;

 public:
  /**
   * @brief Tokens for the root block
   */
  std::vector<Token> root;
  /**
   * @brief Tokens for each block
   */
  Blocks blocks;
  /**
   * @brief Expands blocks tokens of root
   *
   * @warning Can loop forever from mutual recursion from blocks
   */
  void flatten();
  /**
   * @brief Outputs token
   */
  friend std::ostream& operator<<(std::ostream& stream, const Result& result);
};

}  // namespace templating

#endif
