#include "result.h"

#include <gmock/gmock.h>
#include <gtest/gtest.h>

TEST(templatingResult, flatten) {
  templating::Token text1(templating::TokenType::text);
  text1.data[0] = "text1";
  templating::Token text2(templating::TokenType::text);
  text2.data[0] = "text2";
  templating::Token block1(templating::TokenType::block);
  block1.data[0] = "block1";
  templating::Token block2(templating::TokenType::block);
  block2.data[0] = "block2";
  {
    templating::Result result;
    result.root = {text1};
    result.flatten();
    EXPECT_THAT(result.root, testing::ElementsAre(text1));
    EXPECT_TRUE(result.blocks.empty());
  }
  {
    templating::Result result;
    result.root = {text1, block1};
    result.blocks = {{block1.data[0], {text2}}};
    result.flatten();
    EXPECT_THAT(result.root, testing::ElementsAre(text1, text2));
    EXPECT_TRUE(result.blocks.empty());
  }
  {
    templating::Result result;
    result.root = {text1, block1, block1};
    result.blocks = {{block1.data[0], {text2}}};
    result.flatten();
    EXPECT_THAT(result.root, testing::ElementsAre(text1, text2, text2));
    EXPECT_TRUE(result.blocks.empty());
  }
  {
    templating::Result result;
    result.root = {text1, block1};
    result.blocks = {{block1.data[0], {block2}}, {block2.data[0], {text2}}};
    result.flatten();
    EXPECT_THAT(result.root, testing::ElementsAre(text1, text2));
    EXPECT_TRUE(result.blocks.empty());
  }
}