#include "state.h"

#include <cassert>

namespace templating {

State::State(const std::string_view filename, const std::string_view contents)
    : filename(filename), contents(contents), location({0, 1, 1}) {}

std::optional<char> State::peek(const size_t n) const {
  if (contents.length() <= location.cursor + n) {
    return std::nullopt;
  }
  return contents[location.cursor + n];
}

std::optional<char> State::read() {
  if (contents.length() <= location.cursor) {
    return std::nullopt;
  }
  char c = contents[location.cursor];
  location.cursor++;
  // Move location to the next line when we encounter a new line
  if (c == '\n') {
    location.line++;
    location.column = 1;
  } else {
    location.column++;
  }
  return c;
}

bool State::end() const { return contents.length() <= location.cursor; }

void State::push() { history.push_back({location.cursor, location.line, location.column}); }

void State::pop() {
  assert(!history.empty());  // The stack should not be empty
  location = history.back();
  history.pop_back();
}

void State::drop() { history.pop_back(); }

std::string_view State::file() const { return filename; }

size_t State::line() const { return location.line; }

size_t State::column() const { return location.column; }

}  // namespace templating
