#include "state.h"

#include <gtest/gtest.h>

TEST(templatingState, peek) {
  const std::string filename("filename");
  const std::string contents("test");
  const templating::State state(filename, contents);
  EXPECT_EQ(state.peek(), 't');
  EXPECT_EQ(state.peek(0), 't');
  EXPECT_EQ(state.peek(1), 'e');
  EXPECT_EQ(state.peek(2), 's');
  EXPECT_EQ(state.peek(3), 't');
  EXPECT_EQ(state.peek(4), std::nullopt);
}

TEST(templatingState, read) {
  const std::string filename("filename");
  const std::string contents("test");
  templating::State state(filename, contents);
  EXPECT_EQ(state.read(), 't');
  EXPECT_EQ(state.read(), 'e');
  EXPECT_EQ(state.read(), 's');
  EXPECT_EQ(state.read(), 't');
  EXPECT_EQ(state.read(), std::nullopt);
}

TEST(templatingState, end) {
  const std::string filename("filename");
  const std::string contents("test");
  templating::State state(filename, contents);
  EXPECT_EQ(state.end(), false);
  state.read();
  EXPECT_EQ(state.end(), false);
  state.read();
  EXPECT_EQ(state.end(), false);
  state.read();
  EXPECT_EQ(state.end(), false);
  state.read();
  EXPECT_EQ(state.end(), true);
}

TEST(templatingState, pushPushPopPop) {
  const std::string filename("filename");
  const std::string contents("test");
  templating::State state(filename, contents);
  state.read();
  state.push();
  EXPECT_EQ(state.column(), 2);
  state.read();
  state.push();
  EXPECT_EQ(state.column(), 3);
  state.read();
  state.pop();
  EXPECT_EQ(state.column(), 3);
  state.read();
  state.pop();
  EXPECT_EQ(state.column(), 2);
}

TEST(templatingState, pushPushDropPop) {
  const std::string filename("filename");
  const std::string contents("test");
  templating::State state(filename, contents);
  state.read();
  state.push();
  EXPECT_EQ(state.column(), 2);
  state.read();
  state.push();
  EXPECT_EQ(state.column(), 3);
  state.read();
  state.drop();
  EXPECT_EQ(state.column(), 4);
  state.read();
  state.pop();
  EXPECT_EQ(state.column(), 2);
}

TEST(templatingState, fileLineColumn) {
  const std::string filename("filename");
  const std::string contents("t1\nt2");
  templating::State state(filename, contents);
  EXPECT_EQ(state.file(), filename);
  state.read();
  EXPECT_EQ(state.column(), 2);
  EXPECT_EQ(state.line(), 1);
  state.read();
  EXPECT_EQ(state.column(), 3);
  EXPECT_EQ(state.line(), 1);
  state.read();
  EXPECT_EQ(state.column(), 1);
  EXPECT_EQ(state.line(), 2);
  state.read();
  EXPECT_EQ(state.column(), 2);
  EXPECT_EQ(state.line(), 2);
  state.read();
  EXPECT_EQ(state.column(), 3);
  EXPECT_EQ(state.line(), 2);
  state.read();
  EXPECT_EQ(state.column(), 3);
  EXPECT_EQ(state.line(), 2);
}