#include "token.h"

namespace templating {

Token::Token(TokenType type) : type(type) {}

bool Token::operator==(const Token& other) const {
  return type == other.type && data[0] == other.data[0] && data[1] == other.data[1];
}

std::ostream& operator<<(std::ostream& stream, const Token& token) {
  switch (token.type) {
    case TokenType::text:
      return stream << "text: " << token.data[0];
    case TokenType::vspace:
      return stream << "vspace";
    case TokenType::block:
      return stream << "block: " << token.data[0];
    case TokenType::forBegin:
      return stream << "for begin: " << token.data[0] << " in " << token.data[1];
    case TokenType::forEnd:
      return stream << "for end";
    case TokenType::ifBegin:
      return stream << "if begin: " << token.data[0];
    case TokenType::elseBegin:
      return stream << "else begin";
    case TokenType::ifEnd:
      return stream << "if end";
    case TokenType::let:
      return stream << "let: " << token.data[0] << " = " << token.data[1];
    case TokenType::expression:
      return stream << "expression: " << token.data[0];
  }
  return stream;
}

}  // namespace templating
