#ifndef TEMPLATING_TOKEN_H
#define TEMPLATING_TOKEN_H

#include <ostream>

namespace templating {

/**
 * @brief Different types of tokens
 */
enum class TokenType {
  /// Text without verticle spacing
  text,
  /// Verticle space
  vspace,
  /// Name of block to include
  block,
  /// Item and container to use in a for loop
  forBegin,
  /// The end of a for loop
  forEnd,
  /// Expression of any C++ code
  ifBegin,
  /// The beginning of an if block
  elseBegin,
  /// The end of an if block
  ifEnd,
  /// Defines a constant variable
  let,
  /// Expression of any C++ code
  expression
};

/**
 * @brief Parsed token
 */
class Token {
 public:
  explicit Token(TokenType type);
  bool operator==(const Token& other) const;
  /**
   * @brief Outputs token
   */
  friend std::ostream& operator<<(std::ostream& stream, const Token& result);
  /**
   * @brief The type of token
   */
  TokenType type;
  /**
   * @brief Potentially up to two strings worth of related data
   */
  std::string data[2];
};

}  // namespace templating

#endif
