#include "posix_filesystem.h"

#include <sys/stat.h>

#include <iostream>

namespace util {

std::optional<std::string> PosixFilesystem::readFile(const std::string& path) {
  struct stat statBuffer;
  std::string contents;
  FILE* file = fopen(path.c_str(), "r");
  if (!file) {
    if (errno == ENOENT) {
      std::cerr << path << " does not exist" << std::endl;
    } else {
      perror("fopen");
    }
    return std::nullopt;
  }
  if (fstat(fileno(file), &statBuffer) == -1) {
    perror("fstat");
    return std::nullopt;
  }
  contents.resize(statBuffer.st_size);
  if (fread(contents.data(), statBuffer.st_size, 1, file) <= 0 && errno) {
    perror("fread");
    return std::nullopt;
  }
  fclose(file);
  return contents;
}

bool PosixFilesystem::writeFile(const std::string& path, const std::string_view contents) {
  FILE* file = fopen(path.c_str(), "w");
  if (!file) {
    if (errno == ENOENT) {
      std::cerr << path << " could not be created" << std::endl;
    } else {
      perror("fopen");
    }
    return false;
  }
  if (fwrite(contents.data(), contents.length(), 1, file) <= 0 && errno) {
    perror("fwrite");
    return false;
  }
  fclose(file);
  return true;
}

}  // namespace util