#include "util.h"

#include <gtest/gtest.h>

TEST(utilUtil, escapeLiteral) {
  EXPECT_EQ(util::escapeLiteral("", 0), "");
  EXPECT_EQ(util::escapeLiteral("", 4), "");
  EXPECT_EQ(util::escapeLiteral("test\ntest", 0), "test\\n\"\n\"test");
  EXPECT_EQ(util::escapeLiteral("test\ntest", 4), "test\\n\"\n    \"test");
  EXPECT_EQ(util::escapeLiteral("\a\b\t\v\f\r\x1B\"\\", 0),
            "\\a\\b\\t\\v\\f\\r\\x1B\\\"\\\\");
  EXPECT_EQ(util::escapeLiteral("\a\b\t\v\f\r\x1B\"\\", 4),
            "\\a\\b\\t\\v\\f\\r\\x1B\\\"\\\\");
  for (size_t i = 0; i < 256; i++) {
    char c = static_cast<char>(i);
    if (std::string("\a\b\t\n\v\f\r\x1B\"\\").find(c) != std::string::npos) {
      continue;
    }
    std::string s = std::string(1, c);
    EXPECT_EQ(util::escapeLiteral(s, 0), s);
    EXPECT_EQ(util::escapeLiteral(s, 4), s);
  }
  for (size_t i = 0; i < 256; i++) {
    EXPECT_EQ(util::escapeLiteral("test", i), "test");
    EXPECT_EQ(util::escapeLiteral("test\ntest", i).length(),
              strlen("test\\n\"\n\"test") + i);
  }
}

TEST(utilUtil, limit) {
  EXPECT_EQ(util::limit("", 0), "");
  EXPECT_EQ(util::limit("test", 0), "");
  EXPECT_EQ(util::limit("test", 1), "t");
  EXPECT_EQ(util::limit("test", 2), "te");
  EXPECT_EQ(util::limit("test", 3), "tes");
  EXPECT_EQ(util::limit("test", 4), "test");
  EXPECT_EQ(util::limit("test", 5), "test");
}

TEST(utilUtil, stripHTML) {
  EXPECT_EQ(util::stripHTML(""), "");
  EXPECT_EQ(util::stripHTML("<b>test"), "test");
  EXPECT_EQ(util::stripHTML("<b>test</b>test"), "testtest");
  EXPECT_EQ(util::stripHTML("<b><b>te</b>st</b>test"), "testtest");
}

TEST(utilUtil, fileClass) {
  EXPECT_EQ(util::fileClass("png"), util::FileClass::image);
  EXPECT_EQ(util::fileClass("png"), util::fileClass("jpg"));
  EXPECT_EQ(util::fileClass("png"), util::fileClass("gif"));
  EXPECT_EQ(util::fileClass("png"), util::fileClass("webp"));
  EXPECT_EQ(util::fileClass("png"), util::fileClass("svg"));
  EXPECT_EQ(util::fileClass("webm"), util::FileClass::video);
  EXPECT_EQ(util::fileClass("webm"), util::fileClass("mp4"));
  EXPECT_EQ(util::fileClass("mp3"), util::FileClass::audio);
  EXPECT_EQ(util::fileClass("mp3"), util::fileClass("ogg"));
  EXPECT_EQ(util::fileClass("mp3"), util::fileClass("flac"));
  EXPECT_EQ(util::fileClass("pdf"), util::FileClass::pdf);
  EXPECT_EQ(util::fileClass("epub"), util::FileClass::document);
  EXPECT_EQ(util::fileClass("txt"), util::FileClass::text);
  EXPECT_EQ(util::fileClass(""), util::FileClass::unknown);
  EXPECT_EQ(util::fileClass("test"), util::FileClass::unknown);
}

TEST(utilUtil, extension) {
  EXPECT_EQ(util::extension(""), "");
  EXPECT_EQ(util::extension("test"), "test");
  EXPECT_EQ(util::extension("test."), "");
  EXPECT_EQ(util::extension("test.ext"), "ext");
  EXPECT_EQ(util::extension(".ext"), "ext");
}